//--------------------------------------------------------------------------;
//
//  File: ppenum.cpp
//
//  Copyright (c) 2003 Microsoft Corporation.  All rights reserved
//
//  Abstract:
//      This example demonstrates enumerating Plug-n-Play devices in the
//      Windows Preinstallation Environment (WinPE).
//
//  Compiling:
//      Compile with user32.lib, setupapi.lib, and cfgmgr32.lib.
//
//
//--------------------------------------------------------------------------;

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <tchar.h>
#include <strsafe.h>
#include <cfgmgr32.h>

//
// This macro determines the size of a statically-allocated array
//
#define AR_SIZE(a) ( sizeof(a)/sizeof(a[0]) )

//
// Device Class Reg Path
//
#define DEVICE_CLASS_ROOT TEXT("SYSTEM\\CurrentControlSet\\Control\\Class\\")
TCHAR g_RegDeviceInstance[256]=TEXT("");
//
// This global is used to filter the output to either a particular
// service or particular class, depending on its value.
//
//   1: no filter
//   2: filter by service 
//   3: filter by class
//

int g_FilterMode = 1;

//
// Controls how much the text is indented for each increase in depth of the
// tree
//
TCHAR g_IndentSpace[]  =TEXT("  ");

//
// The following two strings are used by the output filtering code to
// determine which class or service to do the filtering on.
//
TCHAR g_szClass[256]=TEXT("");
TCHAR g_szService[256]=TEXT("");


//	
// This global controls an option to print verbose devnode status.
// (off by default)
//
bool g_bVerbose = false;

//
// This global stores the total count of devnodes, not including the root.
//
int g_nDevNodeCount = 0;

//
// This global handle refers to the console that this app
// is running in.
//
HANDLE	g_hStdOut;

#define GREY  0x7
#define GREEN 0xa
#define RED   0xc

//
// Store the default Console Screen Attribute to restore before exit
//
CONSOLE_SCREEN_BUFFER_INFO		g_ConsoleScreen;
//
// Displays the usage message of this application to the console's standard
// output.
//
void
Usage( VOID );

//
// Parses commandline parameters.
//
int
ParseParameters ( IN  int     argc,
                 IN  char*   argv[] );

//
// Recursively traverses the devices in the system
//
VOID
TraverseDevices( IN DEVNODE DevNode, IN int nDepth );

//
// Displays information related to a particular device node.
//
BOOL
DisplayDevNode( IN DEVNODE DevNode, IN int nDepth );

//
// Checks to see if a particular device node should be filtered out or not.
//
BOOL
FilterDevNodeByClassOrService (
                               IN DEVNODE DevNode
                               );

//
// Prints out the status of a device node.
//
VOID 
printDNStatus( 
              ULONG dnStatus,
              ULONG cmProblem,
              int   nDepth
              );

//
// Main procedure of the application
//

int
_cdecl main( int argc, char *argv[] )
{
    int	      nRet = 1;
    CONFIGRET configret;
    DEVNODE   dnRoot;

    nRet = ParseParameters (
        argc,
        argv
        );

    if ( nRet != 1 )
    {
        return 2;
    }

    g_hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);

	if (!GetConsoleScreenBufferInfo ( g_hStdOut, &g_ConsoleScreen))
	{
        _tprintf (
            TEXT("Failed to retrieve console screen attribute (error %d)\n"),
            GetLastError());
		return 2;
	}

    configret = CM_Locate_DevNode(&dnRoot,
        NULL,
        CM_LOCATE_DEVNODE_NORMAL);
    if (CR_SUCCESS != configret) {
        //
        // Querying the root of the device node tree failed, so we can't
        // continue.
        //
        _tprintf (
            TEXT("Can't retrieve the root devnode (error 0x%x)\n"),
            configret);

        return 2;
    } else {

        TraverseDevices(dnRoot, 0);
        _tprintf (TEXT("\r\n\r\nTotal %d DEVNODEs found in the system\r\n"), g_nDevNodeCount);
    }

	SetConsoleTextAttribute (g_hStdOut, g_ConsoleScreen.wAttributes);
    //CloseHandle (g_hStdOut);
    return 1;
}


VOID
TraverseDevices (
                 IN     DEVNODE      DevNode,
                 IN	int nDepth
                 )
/*++

Routine Description

Recursively walks a subtree with the root in
DevNode 

Arguments

DevNode - the root of the processed subtree


Return Value

None.
--*/
{
    DEVNODE   dnChild;
    DEVNODE   dnSibling;
    CONFIGRET configret;

    //
    // Maintain the number of total device nodes
    //
    g_nDevNodeCount++;

    //
    // Print information about the node
    //
    DisplayDevNode(DevNode, nDepth);

    //
    // Go to the child and sibling
    //
    configret = CM_Get_Child(&dnChild, DevNode, 0);

    if (CR_SUCCESS == configret) {
        //
        // Recursively traverse the child subtree
        //
        TraverseDevices(dnChild, nDepth+1);
    }

    configret = CM_Get_Sibling(&dnSibling, DevNode, 0);

    if (CR_SUCCESS == configret) {
        //
        // Recursively traverse the sibling subtree
        //
        TraverseDevices(dnSibling, nDepth);
    }
}

VOID
Usage( VOID )
{
    _tprintf (TEXT("pnpenum.exe [-s service] [-c class] [-v]\r\n\r\n"));   
    _tprintf (TEXT("	[-s <Service Name> | -c <Class Name> ] filter by service, or by class\r\n"));
    _tprintf (TEXT("	  (without -s or -c, display all devices)\r\n"));
    _tprintf (TEXT("	[-v] print device status\r\n"));
    _tprintf (TEXT("	The DEVNODE instance is highlighted in green in the console printout\r\n"));
    _tprintf (TEXT("	The level of indention in the printout indicates the node depth in the dev tree\r\n"));
    _tprintf (TEXT("	\r\nExamples: \r\n"));
    _tprintf (TEXT("	pnpnenum.exe -v //list all devices, including all of their status\r\n"));
    _tprintf (TEXT("	pnpnenum.exe -s PCI //list  devices controlled by PCI service\r\n"));
    _tprintf (TEXT("	pnpnenum.exe -c NET //list  devices belonging to NET class\r\n"));
    _tprintf (TEXT("	pnpnenum.exe -c System //list devices belonging to System class\r\n"));

}

int
ParseParameters ( IN  int     argc, IN  char*   argv[] )
/*++

Return Value:
follow reliant harness report convention
1 for success
2 for failure 

--*/
{
    int i = 1;
    HRESULT hr;

    while (i < argc)
    {
        if ( *argv[i] != '-' && *argv[i] != '/' )
        {
            Usage ();
            return 2;
        }
        switch (argv[i][1])
        { 

		case '?':
            Usage ();
            return 2;

        case 'S':
        case 's':

            //
            // we can only filter by s or c or no filter
            //

            if (g_FilterMode != 1)
            {
                Usage ();
                return 2;
            }
            g_FilterMode = 2;

            //
            // make sure -d is followed by a string and the string
            // doesn't start with '-' or '/"
            //

            if (i+1 >= argc)
            {
                Usage ();
                return 2;
            }

            if ( *argv[i+1] == '-'|| *argv[i+1] == '/' )
            {
                Usage ();
                return 2;
            }

            hr = StringCchCopy(
                g_szService, 
                AR_SIZE( g_szService ), 
                argv[i+1]
                );
                if (FAILED(hr))
                {
                    _tprintf (TEXT("The service name is too long (error 0x%x)\r\n"), hr);
                    Usage ();
                    return 2;
                }
                i += 2;
                break;


        case 'v':
        case 'V':
            g_bVerbose = true;
            i++;
            break;

        case 'C':
        case 'c':


            //
            // we can only filter by s or c or no filter
            //
            if (g_FilterMode != 1)
            {
                Usage ();
                return 2;
            }
            g_FilterMode = 3;
            //
            // make sure -d is followed by a string and the string
            // doesn't start with '-' or '/"
            //

            if (i+1 >= argc)
            {
                Usage ();
                return 2;
            }

            if ( *argv[i+1] == '-'|| *argv[i+1] == '/' )
            {
                Usage ();
                return 2;
            }

            hr = StringCchCopy(g_szClass, AR_SIZE(g_szClass), argv[i+1]);
            if (FAILED(hr))
            {
                _tprintf (TEXT("The class name is too long (error 0x%x)\r\n"), hr);
                Usage ();
                return 2;
            }
            i += 2;
            break;

        default:
            _tprintf (
                TEXT("Invalid command line argument: %s\r\n"),
                argv[i]
                );
                Usage ();
                return 2;

        }
    }

    return 1;

}

BOOL
DisplayDevNode (
                IN DEVNODE DevNode,
                IN	int nDepth
                )
                /*++
                Routine Description

                Checks if the devnode is in good shape and started

                Arguments

                DevNode - the device processed

                Return Value

                TRUE if status is OK, FALSE if not.

                --*/
{
    TCHAR     szRegBuffer[MAX_PATH];
    TCHAR     szDeviceName[MAX_DEVICE_ID_LEN];
    CONFIGRET configret ;
    ULONG     ulStatus;
    ULONG     ulProblem;
    ULONG     ulSize;
    int	      i;

    if ( !FilterDevNodeByClassOrService(DevNode ) ) 
        return FALSE;

    configret = CM_Get_Device_ID(DevNode,
        szDeviceName,
        MAX_DEVICE_ID_LEN,
        0);

    //
    // Change the text color to green
    //
    SetConsoleTextAttribute(g_hStdOut, GREEN);

    if (CR_SUCCESS != configret) {
        _tprintf (
            TEXT("Can't get device name for 0x%x (error 0x%x)\n"),
            DevNode,
            configret
            );
    }
    else
    {
        for ( i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT(TEXT("%s\r\n")), szDeviceName);
    }

    //
    // Change the text color back to grey
    //
    SetConsoleTextAttribute(g_hStdOut, GREY);


    ulSize = sizeof(TCHAR) * MAX_PATH;
    szRegBuffer[0] = TEXT('\0');
    configret = CM_Get_DevNode_Registry_Property(DevNode,
        CM_DRP_DEVICEDESC,
        NULL,
        szRegBuffer,
        &ulSize,
        0);

    if (CR_SUCCESS == configret)
    {
        for (i=0; i<nDepth; i++) _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("DEVICE_DESC: %s\r\n"), szRegBuffer);

    }

    ulSize = sizeof(TCHAR) * MAX_PATH;
    szRegBuffer[0] = TEXT('\0');
    configret = CM_Get_DevNode_Registry_Property(DevNode,
        CM_DRP_HARDWAREID,
        NULL,
        szRegBuffer,
        &ulSize,
        0);
    if (CR_SUCCESS == configret)
    {
        for (i=0; i<nDepth; i++) _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("HARDWAREID: %s\r\n"), szRegBuffer);

    }

    ulSize = sizeof(TCHAR) * MAX_PATH;
    szRegBuffer[0] = TEXT('\0');
    configret = CM_Get_DevNode_Registry_Property(DevNode,
        CM_DRP_COMPATIBLEIDS,
        NULL,
        szRegBuffer,
        &ulSize,
        0);
    if (CR_SUCCESS == configret)
    {
        for (i=0; i<nDepth; i++) _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("COMPATIBLEIDS: %s\r\n"), szRegBuffer);

    }

    ulSize = sizeof(TCHAR) * MAX_PATH;
    szRegBuffer[0] = TEXT('\0');
    configret = CM_Get_DevNode_Registry_Property(DevNode,
        CM_DRP_CLASS,
        NULL,
        szRegBuffer,
        &ulSize,
        0);
    if (CR_SUCCESS == configret)
    {
        for (i=0; i<nDepth; i++) _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("CLASS: %s\r\n"), szRegBuffer);

    }

    ulSize = sizeof(TCHAR) * MAX_PATH;
    szRegBuffer[0] = TEXT('\0');
    configret = CM_Get_DevNode_Registry_Property(DevNode,
        CM_DRP_DRIVER,
        NULL,
        szRegBuffer,
        &ulSize,
        0);
    if (CR_SUCCESS == configret)
    {
        for (i=0; i<nDepth; i++) _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("CLSGUID\\INSTANCE: %s\r\n"), szRegBuffer);

		HKEY	hKey;
		LONG	lRet;
		HRESULT	hr;
		DWORD	dwType;
		TCHAR	szData[MAX_PATH];
		DWORD	dwDataSize = MAX_PATH;

        hr = StringCchCopy(
                g_RegDeviceInstance, 
                AR_SIZE( g_RegDeviceInstance ), 
                DEVICE_CLASS_ROOT
                );
              
		if (FAILED(hr))
        {
			_tprintf (TEXT("StringCchCopy DEVICE_CLASS_ROOT failed (error 0x%x)\r\n"), hr);
            return FALSE;
        }		

        hr = StringCchCat(
                g_RegDeviceInstance, 
                AR_SIZE( g_RegDeviceInstance ), 
                szRegBuffer
                );
              
		if (FAILED(hr))
        {
			_tprintf (TEXT("StringCchCat CLSGuid\\Instance failed (error 0x%x)\r\n"), hr);
            return FALSE;
        }		
		
		lRet = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE, 
				g_RegDeviceInstance,
				0,
				KEY_READ, 
				&hKey
				);
		if(ERROR_SUCCESS != lRet)
		{
			_tprintf (TEXT("RegOpenKeyEx CLSGuid\\Instance failed (error %d)\r\n"),GetLastError());
            return FALSE;
		}
      
		lRet = RegQueryValueEx(
				hKey, 
				TEXT("InfPath"), 
				NULL, 
				&dwType, 
				(BYTE*)szData, 
				&dwDataSize
				);

		if(ERROR_SUCCESS != lRet)
		{
			//_tprintf (TEXT("RegQueryValueEx CLSGuid\\Instance\\InfPath failed (error %d)\r\n"),GetLastError());
		}
		else
		{
			if (dwType == REG_SZ)
			{
				for (i=0; i<nDepth; i++) _tprintf (TEXT("%s"), g_IndentSpace);
				_tprintf (TEXT("InfPath: %s\r\n"), szData);
			}
		}

		lRet = RegQueryValueEx(
				hKey, 
				TEXT("InfSection"), 
				NULL, 
				&dwType, 
				(BYTE*)szData, 
				&dwDataSize
				);

		if(ERROR_SUCCESS != lRet)
		{
			//_tprintf (TEXT("RegQueryValueEx CLSGuid\\Instance\\InfSection failed (error %d)\r\n"),GetLastError());
		}
		else
		{
			if (dwType == REG_SZ)
			{
				for (i=0; i<nDepth; i++) _tprintf (TEXT("%s"), g_IndentSpace);
				_tprintf (TEXT("InfSection: %s\r\n"), szData);
			}
		}

		RegCloseKey (hKey);

    }

    configret = CM_Get_DevNode_Status(&ulStatus,
        &ulProblem,
        DevNode, 
        0);

    if (CR_SUCCESS != configret) 
    {
        //
        // Before logging an error, make sure this devnode is not the root
        // by calculating it's depth in the tree.
        //
        ULONG       ulDepth = 0;

        configret = CM_Get_Depth(&ulDepth, DevNode, 0);

        if ( ( CR_SUCCESS != configret ) ||
            ( 0 != ulDepth ) )
        {
            _tprintf (
                TEXT("Can't retrieve status for %s (error 0x%x)\n"),
                szDeviceName,
                configret
                );
        }
        return FALSE;
    }

    //
    // If the app is in verbose mode, print out the device status.
    //
    if (g_bVerbose)
        printDNStatus(ulStatus, ulProblem, nDepth);

    return TRUE;
}


VOID 
printDNStatus( 
              ULONG dnStatus,
              ULONG cmProblem,
              int nDepth
              )
{
    int i;

    if (DN_ROOT_ENUMERATED & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_ROOT_ENUMERATED");
    }
    if (DN_DRIVER_LOADED & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_DRIVER_LOADED");
    }
    if (DN_ENUM_LOADED & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_ENUM_LOADED");
    }
    if (DN_STARTED & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_STARTED");
    }
    if (DN_MANUAL & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_MANUAL");
    }
    if (DN_NEED_TO_ENUM & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NEED_TO_ENUM");
    }
    if (DN_NOT_FIRST_TIME & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NOT_FIRST_TIME");
    }
    if (DN_HARDWARE_ENUM & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_HARDWARE_ENUM");
    }
    if (DN_LIAR & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NEED_RESTART");
    }
    if (DN_HAS_MARK & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_HAS_MARK");
    }
    if (DN_HAS_PROBLEM & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_HAS_PROBLEM");
    }
    if (DN_FILTERED & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_FILTERED");
    }
    if (DN_MOVED & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_MOVED");
    }
    if (DN_DISABLEABLE & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_DISABLEABLE");
    }
    if (DN_REMOVABLE & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_REMOVABLE");
    }
    if (DN_PRIVATE_PROBLEM & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_PRIVATE_PROBLEM");
    }
    if (DN_MF_PARENT & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_MF_PARENT");
    }
    if (DN_MF_CHILD & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_MF_CHILD");
    }
    if (DN_WILL_BE_REMOVED & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_WILL_BE_REMOVED");
    }
    if (DN_NOT_FIRST_TIMEE & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NOT_FIRST_TIMEE");
    }
    if (DN_STOP_FREE_RES & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_STOP_FREE_RES");
    }
    if (DN_REBAL_CANDIDATE & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_REBAL_CANDIDATE");
    }
    if (DN_BAD_PARTIAL & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_BAD_PARTIAL");
    }
    if (DN_NT_ENUMERATOR & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NT_ENUMERATOR");
    }
    if (DN_NT_DRIVER & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NT_DRIVER");
    }
    if (DN_NEEDS_LOCKING & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NEEDS_LOCKING");
    }
    if (DN_ARM_WAKEUP & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_ARM_WAKEUP");
    }
    if (DN_APM_ENUMERATOR & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_APM_ENUMERATOR");
    }
    if (DN_APM_DRIVER & dnStatus) 
    {
        for (i=0; i<nDepth; i++)
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_APM_DRIVER");
    }
    if (DN_SILENT_INSTALL & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_SILENT_INSTALL");
    }
    if (DN_NO_SHOW_IN_DM & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_NO_SHOW_IN_DM");
    }
    if (DN_BOOT_LOG_PROB & dnStatus) 
    {
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf (TEXT("%s\r\n"), "DN_BOOT_LOG_PROB");
    }
    if (cmProblem !=0)
    {
        SetConsoleTextAttribute(g_hStdOut, RED);
        for (i=0; i<nDepth; i++) 
            _tprintf (TEXT("%s"), g_IndentSpace);
        _tprintf ("CM_PROBLEM %x\r\n", cmProblem);
    }
}

BOOL
FilterDevNodeByClassOrService (
                               IN DEVNODE DevNode
                               )
{
    TCHAR       szBuffer[MAX_PATH];
    CONFIGRET   configret;
    ULONG       ulSize;
    BOOL        bRes = TRUE; //default pass

    if (g_FilterMode == 2) // filter by service 
    {
        ulSize = sizeof(TCHAR) * MAX_PATH;
        szBuffer[0] = TEXT('\0');
        configret = CM_Get_DevNode_Registry_Property(
            DevNode,
            CM_DRP_SERVICE,
            NULL,
            szBuffer,
            &ulSize,
            0
            );  
        if (( CR_SUCCESS == configret )&&
            ( 0 == _tcsicmp(szBuffer, g_szService) ) )
            return TRUE;
        else 
            return FALSE;
    }
    if (g_FilterMode == 3) // filter by class 
    {
        ulSize = sizeof(TCHAR) * MAX_PATH;
        szBuffer[0] = TEXT('\0');
        configret = CM_Get_DevNode_Registry_Property(
            DevNode,
            CM_DRP_CLASS,
            NULL,
            szBuffer,
            &ulSize,
            0
            );  
        if (( CR_SUCCESS == configret ) &&
            ( 0 == _tcsicmp(szBuffer, g_szClass) ))
            return TRUE;
        else
            return FALSE;
    }

    return TRUE;
}
