
/*++


Module Name:

    wmismb.cpp

Abstract:

    Use WMI interface to query instance of MSSmBios_RawSMBiosTables class 
    and optionally dump the SMBIOS tables to a file.

Author:

    Wei Hua (weihua) 17-07-03

Revision History:


Usage:

    wmismbios.exe [-d dumpfile]
    
        [-d dumpfile] is the file path where you want to store the raw SMBIOS table

Exit code:

    1: success
    0: error
    

--*/

// 
//	 _Win32_DCOM pulls in CoInitializeSecurity in objbase.h
//
#ifndef _WIN32_DCOM
#define _WIN32_DCOM
#endif

#include <stdio.h>
#include <objbase.h>
#include <wbemcli.h>
#include <strsafe.h>
#include "wmismb.h"

int __cdecl 
main(
    int argc, 
	char* argv[]
	)
/*++

Return Value:
    1 for success
    0 for failure 

--*/
{

	int	nRet = 1;

	nRet = ParseParameters (
				argc,
				argv, 
				&g_pstrDumpFilePath
				);

	if ( nRet != 1 )
	{
		return 0;
	}

	if (g_pstrDumpFilePath) printf ("The dump file path is %s\r\n\r\n", g_pstrDumpFilePath);

	//
	//	Now let's do the IWebm stuff
	// 
    
	IWbemLocator    *pLocator = NULL;
    IWbemServices   *pNamespace = NULL;
    IEnumWbemClassObject *pEnumSMBIOS = NULL;
    
    HRESULT         hr = S_OK;
    
    BSTR            path = SysAllocString(L"root\\wmi");
    BSTR            className = SysAllocString(L"MSSmBios_RawSMBiosTables");
    
    ULONG           uReturned = 1;

	__try 
	{
		hr = CoInitialize(0);
		if (FAILED(hr))
		{
			nRet = 0;
			printf("CoInitialize returned 0x%x:", hr);
			__leave;
		}

        hr = CoInitializeSecurity   ( 
				NULL, -1, NULL, NULL,
				RPC_C_AUTHN_LEVEL_PKT_PRIVACY, 
				RPC_C_IMP_LEVEL_IMPERSONATE, 
				NULL, 
				EOAC_SECURE_REFS, //change to EOAC_NONE if you change dwAuthnLevel to RPC_C_AUTHN_LEVEL_NONE
				NULL 
				);
    
		if (FAILED(hr))
		{
			nRet = 0;
			printf("CoInitializeSecurity returned 0x%x:", hr);
			__leave;
		}

		hr = CoCreateInstance(
			    CLSID_WbemLocator,
                NULL,
				CLSCTX_INPROC_SERVER,
				IID_IWbemLocator,
				(LPVOID *) &pLocator
				);

		if (FAILED(hr))
		{
			nRet = 0;
			printf("CoCreateInstance CLSID_WbemLocator returned 0x%x:", hr);
			__leave;
		}
            
		hr = pLocator->ConnectServer(path, 
				NULL, 
				NULL, 
				NULL, 
				0, 
				NULL, 
				NULL, 
				&pNamespace
				);

		if(hr != WBEM_S_NO_ERROR)
		{
			nRet = 0;
			printf("pLocator->ConnectServer returned 0x%x:", hr);
			__leave;
		}
            
		hr = pNamespace->CreateInstanceEnum(
				className,          // name of class
				0,
				NULL,
				&pEnumSMBIOS
				);  // pointer to enumerator

		if (FAILED(hr))
		{
			nRet = 0;
			printf("pNamespace->CreateInstanceEnum MSSmBios_RawSMBioTables returned 0x%x:", hr);
			__leave;
		}

		while(uReturned == 1)
		{
        
			IWbemClassObject	*pSmbios = NULL;
			CIMTYPE				type = 0;
			BSTR				propName;
			PUCHAR				pData;
			PSMBIOS_DATA		pSmbiosData = NULL;
      
			hr = pEnumSMBIOS->Next(
					4000,           // timeout in four seconds
					1,          
					&pSmbios,   
					&uReturned		// number obtained: one or zero
					);    

			if (SUCCEEDED(hr) && (uReturned == 1))
			{
				VARIANT pVal;
				VariantInit(&pVal);
				propName = SysAllocString(L"SMBiosData");
    
				hr = pSmbios->Get(
						propName,   // property name 
						0L, 
						&pVal,      // output to this variant 
						&type, 
						NULL);
            
				if (pVal.vt ==  (VT_UI1 | VT_ARRAY) )
				{
					SAFEARRAY   * parray = NULL;
					long        cElements = 0;              

					parray =   V_ARRAY(&pVal);
					cElements = parray->rgsabound[0].cElements;
					pData = (unsigned char *)parray->pvData;

					pSmbiosData = (PSMBIOS_DATA) malloc (sizeof (SMBIOS_DATA) + cElements - 1);
					if (!pSmbiosData )
					{
						nRet = 0;
						printf ("malloc SMBIOS_DATA failed \r\n");
						SysFreeString (propName);
						VariantClear(&pVal);
						pSmbios->Release();
						__leave;

					}

					memcpy (&pSmbiosData->ucSMBiosData[0], pData, cElements);

				}
				SysFreeString (propName);

				VariantClear(&pVal);
				propName = SysAllocString(L"DmiRevision");
    
				hr = pSmbios->Get(
						propName,   // property name 
						0L, 
						&pVal,      // output to this variant 
						&type, 
						NULL);

				if (pVal.vt ==  VT_UI1 )
				{
					pSmbiosData->ucDmiRevision = V_UI1(&pVal);
						printf ("DmiRevision:           %d\r\n", V_UI1(&pVal));
				}   
				SysFreeString (propName);

				VariantClear(&pVal);
				propName = SysAllocString(L"Size");
    
				hr = pSmbios->Get(
						propName,   // property name 
						0L, 
						&pVal,      // output to this variant 
						&type, 
						NULL);

				if (pVal.vt ==  VT_I4 )
				{
					pSmbiosData->uSize = V_I4(&pVal);
						printf ("Size:                  %d\r\n", V_I4(&pVal));
				}   
				SysFreeString (propName);

				VariantClear(&pVal);
				propName = SysAllocString(L"SmbiosMajorVersion");
    
				hr = pSmbios->Get(
						propName,   // property name 
						0L, 
						&pVal,      // output to this variant 
						&type, 
						NULL);

				if (pVal.vt ==  VT_UI1 )
				{
					pSmbiosData->ucSmbiosMajorVersion = V_UI1(&pVal);
						printf ("SmbiosMajorVersion:    %d\r\n", V_UI1(&pVal));
				}   
				SysFreeString (propName);

				VariantClear(&pVal);
				propName = SysAllocString(L"SmbiosMinorVersion");
    
				hr = pSmbios->Get(
						propName,   // property name 
						0L, 
						&pVal,      // output to this variant 
						&type, 
						NULL);

				if (pVal.vt ==  VT_UI1 )
				{
					pSmbiosData->ucSmbiosMinorVersion = V_UI1(&pVal);
						printf ("SmbiosMinorVersion:    %d\r\n", V_UI1(&pVal));
				}   
				SysFreeString (propName);

				VariantClear(&pVal);
				propName = SysAllocString(L"Used20CallingMethod");
    
				hr = pSmbios->Get(
						propName,   // property name 
						0L, 
						&pVal,      // output to this variant 
						&type, 
						NULL);

				if (pVal.vt ==  VT_BOOL )
				{
					pSmbiosData->bUsed20CallingMethod = (UCHAR)V_BOOL(&pVal);
					if (V_BOOL(&pVal)) 
						printf ("Used20CallingMethod:   True\r\n\r\n");
					else 
						printf ("Used20CallingMethod:   False\r\n\r\n"); 
				}
				SysFreeString (propName);           
				VariantClear(&pVal);
				// Done with this object.
				pSmbios->Release();
				
				if (g_pstrDumpFilePath)
				{
					printf ("Dump SMBIOS tables to %s\r\n\r\n", g_pstrDumpFilePath);
					DumpSMBIOSTable (g_pstrDumpFilePath, pSmbiosData);
				}
				
				//
				// DisplaySMBIOSTable only diplay type 0, type 1 and type 2 as example
				//
				
				DisplaySMBIOSTable (pSmbiosData);

				free (pSmbiosData);

			}
		} // end while

	} //end of try block
	__finally 
	{

		if (pEnumSMBIOS)
			pEnumSMBIOS->Release();

		if (pNamespace) 
			pNamespace->Release(); 

		if (pLocator) 
			pLocator->Release(); 


		SysFreeString (path);
		SysFreeString (className);
		CoUninitialize();
	}

	return nRet;


}
LPBYTE
FindSmbiosStructure
    (
    PSMBIOS_DATA    pSmbiosData,
    ULONG           uType,
    PULONG          pLength
    )
{
	LPBYTE  pRet  = NULL;
	DWORD   count = 0;
	DWORD   i;



    for (i = 0; i<pSmbiosData->uSize; )
    {
        //
        // See if the type matches, if so, we may need
        //  to find out about the length of the struct.
        //  If no pLength, just return the start of
        //  struct address.
        //

        if (uType == pSmbiosData->ucSMBiosData[i])
        {
            pRet = &(pSmbiosData->ucSMBiosData[i]);
            if (!pLength)
                return pRet;
        }

        //
        // Look of the end of the struct which is
        //  terminated by a double 0.
        //

        i += pSmbiosData->ucSMBiosData[i+1];

        for (; i<pSmbiosData->uSize; i++)
        {
            if (0 == pSmbiosData->ucSMBiosData[i] &&
                0 == pSmbiosData->ucSMBiosData[i+1])
            {
                i += 2;
                break;
            }
        }

        //
        // If we have found our struct then return
        //  the length as well if required.
        //

        if (pRet)
        {
            if (pLength)
            {
                *pLength = (ULONG)(&(pSmbiosData->ucSMBiosData[i]) - pRet);
            }

            return pRet;
        }

        count++;
    }

    return NULL;
}

void
DisplaySMBIOSTable (
    PSMBIOS_DATA    pSmbiosData
	)
/*++

Purpose: 

  DisplaySMBIOSTable only diplay type 0, type 1 as example. Interested users
  could parse the entire table following SMBIOS spec on 
  http://www.dmtf.org/standards/bios.php.

--*/
{
	PSYSTEMINFO		pSysinfoStruct;
    PBIOSINFO		pBiosInfo;
	char *			pChar;
	ULONG           SysinfoStructLen;
    
	//
	//	Dump type 0
	//

	pBiosInfo = (PBIOSINFO) FindSmbiosStructure(
                            pSmbiosData,
                            0,
                            &SysinfoStructLen
                            );
	printf ("== BIOS Information (Type 0 / Length %d / Handle 0x%xh)\r\n", 
		pBiosInfo->Length,
		pBiosInfo->Handle);

    pChar = (PCHAR)pBiosInfo + pBiosInfo->Length;
    if (pBiosInfo->Vendor)
	{
		printf ("BIOS Vendor:                       %s\r\n", (PCHAR)pChar);
		pChar += lstrlen(pChar) + 1;
	}
	else
	{
		printf ("BIOS Vendor:\r\n");
	}
    if (pBiosInfo->BIOS_Version)
	{
		printf ("BIOS Version:                      %s\r\n", (PCHAR)pChar);
		pChar += lstrlen(pChar) + 1;
	}
	else
	{
		printf ("BIOS Version:\r\n");
	}
	printf ("BIOS Starting Address Segment:     0x%xh\r\n",  pBiosInfo->BIOS_Starting_Address_Segment);   
	if (pBiosInfo->BIOS_Release_Date )
	{
		printf ("BIOS Release Date:                 %s\r\n", (PCHAR)pChar);
		pChar += lstrlen(pChar) + 1;
	}
	else
	{
		printf ("BIOS Release Date:\r\n");
	}
	printf ("BIOS ROM Size:                     %dk\r\n",  (pBiosInfo->BIOS_ROM_Size+1)*64);   
	printf ("BIOS Characteristics:              0x%016hxh\r\n",
		pBiosInfo->BIOS_Characteristics
		);

	printf ("  - ISA Supported:                 %s\r\n", 
		pBiosInfo->BIOS_Characteristics & ISA_SUPPORT ? "Yes" : "No");
	printf ("  - MCA Supported:                 %s\r\n", 
		pBiosInfo->BIOS_Characteristics & MCA_SUPPORT ? "Yes" : "No");
	printf ("  - EISA Supported:                %s\r\n", 
		pBiosInfo->BIOS_Characteristics & EISA_SUPPORT ? "Yes" : "No");
	printf ("  - PCI Supported:                 %s\r\n", 
		pBiosInfo->BIOS_Characteristics & PCI_SUPPORT ? "Yes" : "No");
	printf ("  - PC Card (PCMCIA) Supported:    %s\r\n", 
		pBiosInfo->BIOS_Characteristics & PC_CARD_SUPPORT ? "Yes" : "No");
	printf ("  - PNP Supported:                 %s\r\n", 
		pBiosInfo->BIOS_Characteristics & PNP_SUPPORT ? "Yes" : "No");
	printf ("  - APM Supported:                 %s\r\n", 
		pBiosInfo->BIOS_Characteristics & APM_SUPPORT ? "Yes" : "No");
	printf ("  - BIOS Flash Upgradeable:        %s\r\n", 
		pBiosInfo->BIOS_Characteristics & BIOS_IS_FLASH_UPGRADEABLE ? "Yes" : "No");
	printf ("  - BIOS Shadowing Allowed:        %s\r\n", 
		pBiosInfo->BIOS_Characteristics & BIOS_SHADOWING ? "Yes" : "No");
	printf ("  - VL-VESA Supported:             %s\r\n", 
		pBiosInfo->BIOS_Characteristics & VL_VESA ? "Yes" : "No");
	printf ("  - ESCD Supported:                %s\r\n", 
		pBiosInfo->BIOS_Characteristics & ESCD_SUPPORT ? "Yes" : "No");
	printf ("  - Boot from CD Supported:        %s\r\n", 
		pBiosInfo->BIOS_Characteristics & BOOT_FROM_CD ? "Yes" : "No");
	printf ("  - Selectable Boot Supported:     %s\r\n", 
		pBiosInfo->BIOS_Characteristics & SELECTABLE_BOOT ? "Yes" : "No");
	printf ("  - BIOS ROM Socketed:             %s\r\n", 
		pBiosInfo->BIOS_Characteristics & BIOS_ROM_SOCKETED ? "Yes" : "No");
	printf ("  - Boot From PC Card Supported:   %s\r\n", 
		pBiosInfo->BIOS_Characteristics & BOOT_FROM_PC_CARD ? "Yes" : "No");
	printf ("  - Enhanced Disk Drv Supported:   %s\r\n", 
		pBiosInfo->BIOS_Characteristics & EDD_SPEC_SUPPORTED ? "Yes" : "No");
	printf ("  - INT13H NEC Floppy 1.2MB:       %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_13H_JPN_FLOPPY_NEC_9800_1_2MB ? "Yes" : "No");
	printf ("  - INT13H Toshiba Floppy 1.2MB:   %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_13H_JPN_FLOPPY_TOSHIBA_1_2MB ? "Yes" : "No");
	printf ("  - INT13H 5.25\" 360KB Floppy:     %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_13H_5_25_360KB_FLOPPY ? "Yes" : "No");
	printf ("  - INT13H 5.25\" 1.2MB Floppy:     %s\r\n",
		pBiosInfo->BIOS_Characteristics & INT_13H_5_25_1_2MB_FLOPPY ? "Yes" : "No");
	printf ("  - INT13H 3.5\" 720KB Floppy:      %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_13H_3_5_720KB_FLOPPY ? "Yes" : "No");
	printf ("  - INT13H 3.5\" 2.88MB Floppy:     %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_13H_3_5_2_88MB_FLOPPY ? "Yes" : "No");
	printf ("  - INT5H Print Screen Supported:  %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_5H_PRINT_SCREEN ? "Yes" : "No");
	printf ("  - INT9H 8042 Keyboard Supported: %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_9H_8042_KEYBOARD ? "Yes" : "No");
	printf ("  - INT14H Serial Supported:       %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_14H_SERIAL ? "Yes" : "No");
	printf ("  - INT17H Printer Supported:      %s\r\n",
		pBiosInfo->BIOS_Characteristics & INT_17H_PRINTER ? "Yes" : "No");
	printf ("  - INT10H CGA MONO Supported:     %s\r\n", 
		pBiosInfo->BIOS_Characteristics & INT_10H_CGA_MONO_VIDEO ? "Yes" : "No");
	printf ("  - NEC_PC_98:                     %s\r\n",
		pBiosInfo->BIOS_Characteristics & NEC_PC_98 ? "Yes" : "No");

	if (pBiosInfo->Length - 18) //at least one extension byte
	{
		printf ("BIOS Characteristics Extension 01: %xh\r\n",  pBiosInfo->BIOS_Characteristics_Ext[0]);   
		printf ("  - ACPI Supported:                %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & ACPI_SUPPORT ? "Yes" : "No");
		printf ("  - USB Legacy Supported:          %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & USB_LEGACY_SUPPORT ? "Yes" : "No");
		printf ("  - AGP Supported:                 %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & AGP_SUPPORT ? "Yes" : "No");
		printf ("  - I2O Boot Supported:            %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & I2O_BOOT_SUPPORT ? "Yes" : "No");
		printf ("  - LS-120 Boot Supported:         %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & LS_120_BOOT_SUPPORT ? "Yes" : "No");
		printf ("  - ATAPI ZIP Drv Boot Supported:  %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & ATAPI_ZIP_DRIVE_BOOT_SUPPORT ? "Yes" : "No");
		printf ("  - IEEE1394 Boot Supported:       %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & IEEE1394_BOOT_SUPPORT ? "Yes" : "No");
		printf ("  - Smart Battery Supported:       %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[0] & SMART_BATTERY_SUPPORT ? "Yes" : "No");
	}
	if (pBiosInfo->Length - 19) //we have one more extension byte
	{
		printf ("BIOS Characteristics Extension 02: %xh\r\n",  pBiosInfo->BIOS_Characteristics_Ext[1]);   
		printf ("  - BIOS Boot Spec Supported:      %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[1] & BIOS_BOOT_SPECIFICATION ? "Yes" : "No");
		printf ("  - Function Key Network Boot:     %s\r\n", 
			pBiosInfo->BIOS_Characteristics_Ext[1] & FUNCTION_KEY_NETWORK_BOOT ? "Yes" : "No");
	}
	printf ("\r\n");

	//
	//	Dump type 1
	//

	pSysinfoStruct = (PSYSTEMINFO) FindSmbiosStructure(
                            pSmbiosData,
                            1,
                            &SysinfoStructLen
                            );
	printf ("== System Information (Type 1 / Length %d / Handle 0x%xh)\r\n", 
		pSysinfoStruct->Length,
		pSysinfoStruct->Handle);
	
    pChar = (PCHAR)pSysinfoStruct + pSysinfoStruct->Length;
    if (pSysinfoStruct->Manufacturer )
	{
		printf ("Manufacturer:  %s\r\n", (PCHAR)pChar);
		pChar += lstrlen(pChar) + 1;
	}
	else
	{
		printf ("Manufacturer:\r\n");
	}
    if (pSysinfoStruct->Product_Name )
	{
		printf ("Product Name:  %s\r\n", (PCHAR)pChar);
		pChar += lstrlen(pChar) + 1;
	}
	else
	{
		printf ("Product Name:\r\n");
	}
    if (pSysinfoStruct->Version )
	{
		printf ("Version:       %s\r\n", (PCHAR)pChar);
		pChar += lstrlen(pChar) + 1;
	}
	else
	{
		printf ("Version:\r\n");
	}
    if (pSysinfoStruct->Serial_Number )
	{
		printf ("Serial Number: %s\r\n", (PCHAR)pChar);
		pChar += lstrlen(pChar) + 1;
	}
	else
	{
		printf ("Serial Number:\r\n");
	}
    printf ("UUID:          ");
    for (int k=0; k<16; k++)
       printf ("%x ", pSysinfoStruct->UUID[k]);
    printf ("\r\n");
	printf ("Wake-up Type:  ");
	switch (pSysinfoStruct->Wakeup_Type)
	{
		case WU_RESERVED:
			printf ("Reserved\r\n\r\n");
			break;
		case WU_OTHER:
			printf ("Other\r\n\r\n");
			break;
		case WU_UNKNOWN:
			printf ("Unknown\r\n\r\n");
			break;
		case WU_APM_TIMER:
			printf ("APM Timer\r\n\r\n");
			break;
		case WU_MODEM_RING:
			printf ("MODEM RING\r\n\r\n");
			break;
		case WU_LAN_REMOTE:
			printf ("Lan Remote\r\n\r\n");
			break;
		case WU_POWER_SWITCH:
			printf ("Power Switch\r\n\r\n");
			break;
		case WU_PCI_PME:
			printf ("PCI PME\r\n\r\n");
			break;
		case WU_AC_POWER_RESTORED:
			printf ("AC Power Restored\r\n\r\n");
			break;		
		default:
			printf ("Not defined\r\n\r\n");
			break;
	}

}

void
DumpSMBIOSTable (
    LPSTR           pszOutFile,
    PSMBIOS_DATA    pSmbiosData
    )
{
	HANDLE  hDumpFile;
	BOOL    bRet;
	DWORD   dwBytesWritten;

	DUMPFILE_HEAD   DumpFileHead;


    hDumpFile = CreateFile(
                   pszOutFile,
                   GENERIC_WRITE,
                   FILE_SHARE_READ,
                   NULL,
                   CREATE_ALWAYS,
                   FILE_ATTRIBUTE_NORMAL,
                   NULL
                   );

    if (INVALID_HANDLE_VALUE == hDumpFile)
    {
        printf ("CreateFile failed %d\r\n", GetLastError());
        return;
    }

    DumpFileHead.ucSmbiosMajorVersion = pSmbiosData->ucSmbiosMajorVersion;
    DumpFileHead.ucSmbiosMinorVersion = pSmbiosData->ucSmbiosMinorVersion;
    DumpFileHead.ucDmiRevision        = pSmbiosData->ucDmiRevision;


    bRet = WriteFile (
		     hDumpFile,
             &DumpFileHead,
             sizeof (DumpFileHead),
             &dwBytesWritten,
             NULL
			 );

    if (!bRet)
    {
        printf ("WriteFile failed %d\r\n", GetLastError());
        return;
    }


    bRet = WriteFile (
		     hDumpFile,
             pSmbiosData->ucSMBiosData,
             pSmbiosData->uSize,
             &dwBytesWritten,
             NULL
			 );

    if (!bRet)
    {
        printf ("WriteFile failed %d\r\n", GetLastError());
        return;
    }

    CloseHandle (hDumpFile);
}

void
Usage(
    void
    )
{
    printf ("wmismbios.exe [-d dumpfile]\r\n\r\n");
    
    printf ("	[-d dumpfile] file path to dump the raw SMBIOS table\r\n");

}

int
ParseParameters (
    IN  int     argc,
    IN  char*   argv[],
    IN OUT char**   ppstrDumpFile
    )
/*++

Return Value:
    follow reliant harness report convention
    1 for success
    2 for failure 

--*/
{
    int i = 1;
	HRESULT hr;
    
	*ppstrDumpFile = NULL;
	
    while (i < argc)
    {
        if ( *argv[i] != '-' && *argv[i] != '/' )
        {
            Usage ();
            return 0;
        }
        switch (argv[i][1])
        { 
            case 'D':
            case 'd':
		
				//
				// make sure -d is followed by a string and the string doesn't start
				// with '-' or '/"
				//

				if (i+1 >= argc)
				{
					Usage ();
					return 0;
				}
                
				if ( *argv[i+1] == '-'|| *argv[i+1] == '/' )
				{
					Usage ();
					return 0;
				}

				hr = StringCchCopy(g_strDumpFilePath, 256, argv[i+1]);
				if (FAILED(hr))
				{
					printf ("Wrong path StringCchCopy failed 0x%x\r\n", hr);
				    Usage ();
					return 0;
				}
                *ppstrDumpFile = g_strDumpFilePath;
				i += 2;
            break;


			default:
                printf ("Invalid command line argument: %s\r\n", argv[i]);
				Usage ();
				return 0;

		}
    }

	return 1;

}
      
